@@ -1,3 +1,4 @@
+{-# LANGUAGE ScopedTypeVariables, StandaloneDeriving #-}
 {-
 Copyright (C) 2009 John MacFarlane <jgm@berkeley.edu>,
                    Henry Laxen <nadine.and.henry@pobox.com>
@@ -20,7 +21,10 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 {- Handlers for registering and authenticating users.
 -}
 
-module Network.Gitit.Authentication (formAuthHandlers, httpAuthHandlers, loginUserForm) where
+module Network.Gitit.Authentication ( loginUserForm
+                                    , formAuthHandlers
+                                    , httpAuthHandlers
+                                    , rpxAuthHandlers) where
 
 import Network.Gitit.State
 import Network.Gitit.Types
@@ -32,18 +36,21 @@ import Network.Captcha.ReCaptcha (captchaFields, validateCaptcha)
 import Text.XHtml hiding ( (</>), dir, method, password, rev )
 import qualified Text.XHtml as X ( password )
 import System.Process (readProcessWithExitCode)
-import Control.Monad (unless, liftM)
+import Control.Monad (unless, liftM, mplus)
 import Control.Monad.Trans (MonadIO(), liftIO)
 import System.Exit
 import System.Log.Logger (logM, Priority(..))
 import Data.Char (isAlphaNum, isAlpha, isAscii)
+import qualified Data.Map as M
 import Text.Pandoc.Shared (substitute)
-import Data.Maybe (isJust, fromJust)
+import Data.Maybe (isJust, fromJust, isNothing, fromMaybe)
 import Network.URL (encString, exportURL, add_param, importURL)
 import Network.BSD (getHostName)
 import qualified Text.StringTemplate as T
-import Network.HTTP (urlEncodeVars)
-import Codec.Binary.UTF8.String (encodeString) 
+import Network.HTTP (urlEncodeVars, urlDecode, urlEncode)
+import Codec.Binary.UTF8.String (encodeString)
+import Data.ByteString.UTF8 (toString)
+import Network.Gitit.Rpxnow as R
 
 data ValidationType = Register
                     | ResetPassword
@@ -111,7 +118,7 @@ resetPasswordRequest params = do
                     response
     else registerForm >>=
          formattedPage defaultPageLayout{
-                         pgMessages = errors, 
+                         pgMessages = errors,
                          pgShowPageTools = False,
                          pgTabs = [],
                          pgTitle = "Register for an account"
@@ -155,7 +162,7 @@ validateReset params postValidate = do
   let errors = case (knownUser, resetCodeMatches) of
                      (True, True)   -> []
                      (True, False)  -> ["Your reset code is invalid"]
-                     (False, _)     -> ["User " ++ uname ++ " is not known"] 
+                     (False, _)     -> ["User " ++ uname ++ " is not known"]
   if null errors
      then postValidate (fromJust user)
      else registerForm >>=
@@ -366,7 +373,7 @@ loginUser params = do
   if allowed
     then do
       key <- newSession (SessionData uname)
-      addCookie (sessionTimeout cfg) (mkCookie "sid" (show key))
+      addCookie (MaxAge $ sessionTimeout cfg) (mkCookie "sid" (show key))
       seeOther (encUrl destination) $ toResponse $ p << ("Welcome, " ++ uname)
     else
       withMessages ["Invalid username or password."] loginUserForm
@@ -383,8 +390,7 @@ logoutUser params = do
   case key of
        Just k  -> do
          delSession k
-         -- make cookie expire immediately, effectively deleting it
-         addCookie 0 (mkCookie "sid" "-1")
+         expireCookie "sid"
        Nothing -> return ()
   seeOther (encUrl dest) $ toResponse "You have been logged out."
 
@@ -407,6 +413,7 @@ formAuthHandlers =
   , dir "_resetPassword"   $ methodSP POST $ withData resetPasswordRequest
   , dir "_doResetPassword" $ methodSP GET  $ withData resetPassword
   , dir "_doResetPassword" $ methodSP POST $ withData doResetPassword
+  , dir "_user" currentUser
   ]
 
 loginUserHTTP :: Params -> Handler
@@ -421,4 +428,72 @@ logoutUserHTTP = unauthorized $ toResponse ()  -- will this work?
 httpAuthHandlers :: [Handler]
 httpAuthHandlers =
   [ dir "_logout" $ logoutUserHTTP
-  , dir "_login"  $ withData loginUserHTTP ]
+  , dir "_login"  $ withData loginUserHTTP
+  , dir "_user" currentUser ]
+
+-- Login using RPX (see RPX development docs at https://rpxnow.com/docs)
+loginRPXUser :: RPars  -- ^ The parameters passed by the RPX callback call (after authentication has taken place
+             -> Handler
+loginRPXUser params = do
+  cfg <- getConfig
+  ref <- getReferer
+  let mtoken = rToken params
+  if isNothing mtoken
+     then do
+       let url = baseUrl cfg ++ "/_login?destination=" ++
+                  (fromMaybe ref $ rDestination params)
+       if null (rpxDomain cfg)
+          then error "rpx-domain is not set."
+          else do
+             let rpx = "https://" ++ rpxDomain cfg ++
+                       ".rpxnow.com/openid/v2/signin?token_url=" ++
+                       urlEncode url
+             see rpx
+     else do -- We got an answer from RPX, this might also return an exception.
+       uid' :: Either String R.Identifier <- liftIO $
+                      R.authenticate (rpxKey cfg) $ fromJust mtoken
+       uid <- case uid' of
+                   Right u -> return u
+                   Left err -> error err
+       liftIO $ logM "gitit.loginRPXUser" DEBUG $ "uid:" ++ show uid
+       -- We need to get an unique identifier for the user
+       -- The 'identifier' is always present but can be rather cryptic
+       -- The 'verifiedEmail' is also unique and is a more readable choice
+       -- so we use it if present.
+       let userId = R.userIdentifier uid
+       let email  = prop "verifiedEmail" uid
+       user <- liftIO $ mkUser (fromMaybe userId email) (fromMaybe "" email) "none"
+       updateGititState $ \s -> s { users = M.insert userId user (users s) }
+       key <- newSession (SessionData userId)
+       addCookie (MaxAge $ sessionTimeout cfg) (mkCookie "sid" (show key))
+       see $ fromJust $ rDestination params
+      where
+        prop pname info = lookup pname $ R.userData info
+        see url = seeOther (encUrl url) $ toResponse noHtml
+
+-- The parameters passed by the RPX callback call.
+data RPars = RPars { rToken       :: Maybe String
+                   , rDestination :: Maybe String }
+                   deriving Show
+
+instance FromData RPars where
+     fromData = do
+         vtoken <- liftM Just (look "token") `mplus` return Nothing
+         vDestination <- liftM (Just . urlDecode) (look "destination") `mplus`
+                           return Nothing
+         return RPars { rToken = vtoken
+                      , rDestination = vDestination }
+
+rpxAuthHandlers :: [Handler]
+rpxAuthHandlers =
+  [ dir "_logout" $ methodSP GET $ withData logoutUser
+  , dir "_login"  $ withData loginRPXUser
+  , dir "_user" currentUser ]
+
+-- | Returns username of logged in user or null string if nobody logged in.
+currentUser :: Handler
+currentUser = do
+  req <- askRq
+  ok $ toResponse $ maybe "" toString (getHeader "REMOTE_USER" req)
+
+
