@@ -27,7 +27,7 @@ where
 import Network.Gitit.Types
 import Network.Gitit.Server (mimeTypes)
 import Network.Gitit.Framework
-import Network.Gitit.Authentication (formAuthHandlers, httpAuthHandlers)
+import Network.Gitit.Authentication (formAuthHandlers, rpxAuthHandlers, httpAuthHandlers)
 import Network.Gitit.Util (parsePageType, readFileUTF8)
 import System.Log.Logger (logM, Priority(..))
 import qualified Data.Map as M
@@ -38,7 +38,7 @@ import Data.List (intercalate)
 import Data.Char (toLower, toUpper, isDigit)
 import Paths_gitit (getDataFileName)
 import System.FilePath ((</>))
-import Text.Pandoc hiding (MathML)
+import Text.Pandoc hiding (MathML, WebTeX)
 
 forceEither :: Show e => Either e a -> a
 forceEither = either (error . show) id
@@ -66,6 +66,7 @@ extractConfig cp = do
       cfDefaultPageType <- get cp "DEFAULT" "default-page-type"
       cfMathMethod <- get cp "DEFAULT" "math"
       cfShowLHSBirdTracks <- get cp "DEFAULT" "show-lhs-bird-tracks"
+      cfRequireAuthentication <- get cp "DEFAULT" "require-authentication"
       cfAuthenticationMethod <- get cp "DEFAULT" "authentication-method"
       cfUserFile <- get cp "DEFAULT" "user-file"
       cfSessionTimeout <- get cp "DEFAULT" "session-timeout"
@@ -88,6 +89,8 @@ extractConfig cp = do
       cfUseRecaptcha <- get cp "DEFAULT" "use-recaptcha"
       cfRecaptchaPublicKey <- get cp "DEFAULT" "recaptcha-public-key"
       cfRecaptchaPrivateKey <- get cp "DEFAULT" "recaptcha-private-key"
+      cfRPXDomain <- get cp "DEFAULT" "rpx-domain"
+      cfRPXKey <- get cp "DEFAULT" "rpx-key"
       cfCompressResponses <- get cp "DEFAULT" "compress-responses"
       cfUseCache <- get cp "DEFAULT" "use-cache"
       cfCacheDir <- get cp "DEFAULT" "cache-dir"
@@ -115,6 +118,8 @@ extractConfig cp = do
                         "darcs"     -> Darcs
                         "mercurial" -> Mercurial
                         x           -> error $ "Unknown repository type: " ++ x
+      when (authMethod == "rpx" && cfRPXDomain == "") $
+         liftIO $ logM "gitit" WARNING $ "rpx-domain is not set"
 
       return $! Config{
           repositoryPath       = cfRepositoryPath
@@ -123,16 +128,25 @@ extractConfig cp = do
         , mathMethod           = case map toLower cfMathMethod of
                                       "jsmath"   -> JsMathScript
                                       "mathml"   -> MathML
+                                      "google"   -> WebTeX "http://chart.apis.google.com/chart?cht=tx&chl="
                                       _          -> RawTeX
         , defaultLHS           = lhs
         , showLHSBirdTracks    = cfShowLHSBirdTracks
         , withUser             = case authMethod of
                                       "form"     -> withUserFromSession
                                       "http"     -> withUserFromHTTPAuth
+                                      "rpx"      -> withUserFromSession
                                       _          -> id
+        , requireAuthentication = case map toLower cfRequireAuthentication of
+                                       "none"    -> Never
+                                       "modify"  -> ForModify
+                                       "read"    -> ForRead
+                                       _         -> ForModify
+
         , authHandler          = case authMethod of
                                       "form"     -> msum formAuthHandlers
                                       "http"     -> msum httpAuthHandlers
+                                      "rpx"      -> msum rpxAuthHandlers
                                       _          -> mzero
         , userFile             = cfUserFile
         , sessionTimeout       = readNumber "session-timeout" cfSessionTimeout * 60  -- convert minutes -> seconds
@@ -161,6 +175,8 @@ extractConfig cp = do
         , useRecaptcha         = cfUseRecaptcha
         , recaptchaPublicKey   = cfRecaptchaPublicKey
         , recaptchaPrivateKey  = cfRecaptchaPrivateKey
+        , rpxDomain            = cfRPXDomain
+        , rpxKey               = cfRPXKey
         , compressResponses    = cfCompressResponses
         , useCache             = cfUseCache
         , cacheDir             = cfCacheDir
