@@ -48,13 +48,11 @@ module Network.Gitit.Handlers (
                       , deletePage
                       , confirmDelete
                       , showHighlightedSource
-                      , currentUser
                       , expireCache
                       , feedHandler
                       )
 where
 import Safe
-import Data.FileStore.Utils (isInsideDir)
 import Network.Gitit.Server
 import Network.Gitit.Framework
 import Network.Gitit.Layout
@@ -66,7 +64,6 @@ import Network.Gitit.ContentTransformer (showRawPage, showFileAsText, showPage,
         exportPage, showHighlightedSource, preview, applyPreCommitPlugins)
 import Network.Gitit.Page (extractCategories)
 import Control.Exception (throwIO, catch, try)
-import Data.ByteString.UTF8 (toString)
 import System.Time
 import System.FilePath
 import Prelude hiding (catch)
@@ -81,7 +78,7 @@ import Control.Monad.Reader
 import qualified Data.ByteString.Lazy as B
 import qualified Data.ByteString as S
 import Network.HTTP (urlEncodeVars)
-import Data.DateTime (getCurrentTime, addMinutes)
+import Data.Time (getCurrentTime, addUTCTime)
 import Data.FileStore
 import System.Log.Logger (logM, Priority(..))
 
@@ -98,7 +95,7 @@ handleAny = uriRest $ \uri ->
                                   (ok $ setContentType mimetype $
                                     (toResponse noHtml) {rsBody = contents})
                                     -- ugly hack
-                Left NotFound  -> anyRequest mzero
+                Left NotFound  -> mzero
                 Left e         -> error (show e)
 
 debugHandler :: Handler
@@ -182,13 +179,15 @@ uploadForm = withData $ \(params :: Params) -> do
 uploadFile :: Handler
 uploadFile = withData $ \(params :: Params) -> do
   let origPath = pFilename params
-  let fileContents = pFileContents params
-  let wikiname = pWikiname params `orIfNull` takeFileName origPath
+  let filePath = pFilePath params
+  let wikiname = normalise
+                 $ dropWhile (=='/')
+                 $ pWikiname params `orIfNull` takeFileName origPath
   let logMsg = pLogMsg params
   cfg <- getConfig
   mbUser <- getLoggedInUser
   (user, email) <- case mbUser of
-                        Nothing -> fail "User must be logged in to delete page."
+                        Nothing -> return ("Anonymous", "")
                         Just u  -> return (uUsername u, uEmail u)
   let overwrite = pOverwrite params
   fs <- getFileStore
@@ -196,31 +195,31 @@ uploadFile = withData $ \(params :: Params) -> do
                       if e == NotFound
                          then return False
                          else throwIO e >> return True
-  inStaticDir <- liftIO $
-                  (repositoryPath cfg </> wikiname) `isInsideDir` staticDir cfg
-  inTemplatesDir <- liftIO $
-                  (repositoryPath cfg </> wikiname) `isInsideDir` templatesDir cfg
+  let inStaticDir = staticDir cfg `isPrefixOf` (repositoryPath cfg </> wikiname)
+  let inTemplatesDir = templatesDir cfg `isPrefixOf` (repositoryPath cfg </> wikiname)
+  let dirs' = splitDirectories $ takeDirectory wikiname
   let imageExtensions = [".png", ".jpg", ".gif"]
   let errors = validate
                  [ (null . filter (not . isSpace) $ logMsg,
                     "Description cannot be empty.")
+                 , (".." `elem` dirs', "Wikiname cannot contain '..'")
                  , (null origPath, "File not found.")
                  , (inStaticDir,  "Destination is inside static directory.")
                  , (inTemplatesDir,  "Destination is inside templates directory.")
                  , (not overwrite && exists, "A file named '" ++ wikiname ++
                     "' already exists in the repository: choose a new name " ++
                     "or check the box to overwrite the existing file.")
-                 , (B.length fileContents > fromIntegral (maxUploadSize cfg),
-                    "File exceeds maximum upload size.")
                  , (isPageFile wikiname,
                     "This file extension is reserved for wiki pages.")
                  ]
   if null errors
      then do
        expireCachedFile wikiname `mplus` return ()
+       fileContents <- liftIO $ B.readFile filePath
+       let len = B.length fileContents
        liftIO $ save fs wikiname (Author user email) logMsg fileContents
        let contents = thediv <<
-             [ h2 << ("Uploaded " ++ show (B.length fileContents) ++ " bytes")
+             [ h2 << ("Uploaded " ++ show len ++ " bytes")
              , if takeExtension wikiname `elem` imageExtensions
                   then p << "To add this image to a page, use:" +++
                        pre << ("![alt text](/" ++ wikiname ++ ")")
@@ -255,11 +254,11 @@ goToPage = withData $ \(params :: Params) -> do
                                        Just m  -> seeOther (base' ++ urlForPage m) $
                                                   toResponse $ "Redirecting" ++
                                                     " to partial match"
-                                       Nothing -> withInput "patterns" gotopage searchResults
+                                       Nothing -> searchResults
 
 searchResults :: Handler
 searchResults = withData $ \(params :: Params) -> do
-  let patterns = pPatterns params
+  let patterns = pPatterns params `orIfNull` [pGotoPage params]
   fs <- getFileStore
   matchLines <- if null patterns
                    then return []
@@ -288,7 +287,7 @@ searchResults = withData $ \(params :: Params) -> do
   let relevance (f, ms) = length ms + if f `elem` pageNameMatches
                                          then 100
                                          else 0
-  let preamble = if null patterns 
+  let preamble = if null patterns
                     then h3 << ["Please enter a search term."]
                     else h3 << [ stringToHtml (show (length matches) ++ " matches found for ")
                                , thespan ! [identifier "pattern"] << unwords patterns]
@@ -326,7 +325,7 @@ showFileHistory = withData $ \(params :: Params) -> do
 showHistory :: String -> String -> Params -> Handler
 showHistory file page params =  do
   currTime <- liftIO getCurrentTime
-  let oneYearAgo = addMinutes (-1 * 60 * 24 * 365) currTime
+  let oneYearAgo = addUTCTime (-60 * 60 * 24 * 365) currTime
   let since = case pSince params of
                    Nothing -> Just oneYearAgo
                    Just t  -> Just t
@@ -379,7 +378,7 @@ showHistory file page params =  do
 showActivity :: Handler
 showActivity = withData $ \(params :: Params) -> do
   currTime <- liftIO getCurrentTime
-  let oneMonthAgo = addMinutes (-1 * 60 * 24 * 30) currTime
+  let oneMonthAgo = addUTCTime (-60 * 60 * 24 * 30) currTime
   let since = case pSince params of
                    Nothing -> Just oneMonthAgo
                    Just t  -> Just t
@@ -444,7 +443,7 @@ showDiff file page params = do
   from' <- case (from, to) of
               (Just _, _)        -> return from
               (Nothing, Nothing) -> return from
-              (Nothing, Just t)  -> do 
+              (Nothing, Just t)  -> do
                 pageHist <- liftIO $ history fs [file]
                                      (TimeRange Nothing Nothing)
                 let (_, upto) = break (\r -> idsMatch fs (revId r) t)
@@ -480,7 +479,10 @@ getDiff fs file from to = do
            pre ! [theclass "diff"] << map diffLineToHtml rawDiff
 
 editPage :: Handler
-editPage = withData $ \(params :: Params) -> do
+editPage = withData editPage'
+
+editPage' :: Params -> Handler
+editPage' params = do
   let rev = pRevision params  -- if this is set, we're doing a revert
   fs <- getFileStore
   page <- getPage
@@ -533,6 +535,11 @@ editPage = withData $ \(params :: Params) -> do
                               value "Preview" ]
                    , thediv ! [ identifier "previewpane" ] << noHtml
                    ]
+  let pgScripts' = ["preview.js"]
+  let pgScripts'' = case mathMethod cfg of
+       JsMathScript -> "jsMath/easy/load.js" : pgScripts'
+       MathML       -> "MathMLinHTML.js" : pgScripts'
+       _            -> pgScripts'
   formattedPage defaultPageLayout{
                   pgPageName = page,
                   pgMessages = messages,
@@ -541,7 +548,7 @@ editPage = withData $ \(params :: Params) -> do
                   pgShowSiteNav = False,
                   pgMarkupHelp = Just $ markupHelp cfg,
                   pgSelectedTab = EditTab,
-                  pgScripts = ["preview.js"],
+                  pgScripts = pgScripts'',
                   pgTitle = ("Editing " ++ page)
                   } editForm
 
@@ -581,12 +588,12 @@ deletePage = withData $ \(params :: Params) -> do
   let file = pFileToDelete params
   mbUser <- getLoggedInUser
   (user, email) <- case mbUser of
-                        Nothing -> fail "User must be logged in to delete."
+                        Nothing -> return ("Anonymous", "")
                         Just u  -> return (uUsername u, uEmail u)
   let author = Author user email
   let descrip = "Deleted using web interface."
   base' <- getWikiBase
-  if pConfirm params && (file == page || file == page <.> "page") 
+  if pConfirm params && (file == page || file == page <.> "page")
      then do
        fs <- getFileStore
        liftIO $ delete fs file author descrip
@@ -599,7 +606,7 @@ updatePage = withData $ \(params :: Params) -> do
   cfg <- getConfig
   mbUser <- getLoggedInUser
   (user, email) <- case mbUser of
-                        Nothing -> fail "User must be logged in to delete page."
+                        Nothing -> return ("Anonymous", "")
                         Just u  -> return (uUsername u, uEmail u)
   editedText <- case pEditedText params of
                      Nothing -> error "No body text in POST request"
@@ -635,9 +642,10 @@ updatePage = withData $ \(params :: Params) -> do
                       if conflicts
                          then "Please resolve conflicts and Save."
                          else "Please review and Save."
-               withMessages [mergeMsg] $
-                 withInput "editedText" mergedText $
-                   withInput "sha1" (revId mergedWithRev) editPage
+               editPage' $
+                 params{ pEditedText = Just mergedText,
+                         pSHA1       = revId mergedWithRev,
+                         pMessages   = [mergeMsg] }
 
 indexPage :: Handler
 indexPage = do
@@ -730,12 +738,6 @@ categoryListPage = do
                   pgScripts = ["search.js"],
                   pgTitle = "Categories" } htmlMatches
 
--- | Returns username of logged in user or null string if nobody logged in.
-currentUser :: Handler
-currentUser = do
-  req <- askRq
-  ok $ toResponse $ maybe "" toString (getHeader "REMOTE_USER" req)
-
 expireCache :: Handler
 expireCache = do
   page <- getPage
@@ -776,6 +778,6 @@ feedHandler = do
             ok $ emptyResponse{rsBody = B.fromChunks [contents]}
        _ -> do
             fs <- getFileStore
-            resp <- liftM toResponse $ liftIO (filestoreToXmlFeed fc fs mbPath)
-            cacheContents file $ S.concat $ B.toChunks $ rsBody resp
-            ok . setContentType "application/atom+xml; charset=UTF-8" $ resp
+            resp' <- liftM toResponse $ liftIO (filestoreToXmlFeed fc fs mbPath)
+            cacheContents file $ S.concat $ B.toChunks $ rsBody resp'
+            ok . setContentType "application/atom+xml; charset=UTF-8" $ resp'
