@@ -32,6 +32,8 @@ import System.Environment
 import System.Exit
 import System.IO (stderr)
 import System.Console.GetOpt
+import Network.Socket hiding (Debug)
+import Network.URI
 import Data.Version (showVersion)
 import qualified Data.ByteString as B
 import Data.ByteString.UTF8 (fromString)
@@ -74,10 +76,18 @@ main = do
   -- initialize state
   initializeGititState conf'
 
-  let serverConf = Conf { validator = Nothing, port = portNumber conf' }
+  let serverConf = Conf { validator = Nothing, port = portNumber conf',
+                          timeout = 20, logAccess = Nothing }
+
+  -- open the requested interface
+  sock <- socket AF_INET Stream defaultProtocol
+  setSocketOption sock ReuseAddr 1
+  device <- inet_addr (getListenOrDefault opts)
+  bindSocket sock (SockAddrInet (toEnum (portNumber conf')) device)
+  listen sock 10
 
   -- start the server
-  simpleHTTP serverConf $ msum [ wiki conf'
+  simpleHTTPWithSocket sock serverConf $ msum [ wiki conf'
                                , dir "_reloadTemplates" reloadTemplates
                                ]
 
@@ -85,6 +95,7 @@ data Opt
     = Help
     | ConfigFile FilePath
     | Port Int
+	| Listen String
     | Debug
     | Version
     | PrintDefaultConfig
@@ -98,6 +109,8 @@ flags =
         "Print version information"
    , Option ['p'] ["port"] (ReqArg (Port . read) "PORT")
         "Specify port"
+   , Option ['l'] ["listen"] (ReqArg (Listen . checkListen) "INTERFACE")
+        "Specify interface to listen on"
    , Option [] ["print-default-config"] (NoArg PrintDefaultConfig)
         "Print default configuration"
    , Option [] ["debug"] (NoArg Debug)
@@ -106,6 +119,16 @@ flags =
         "Specify configuration file"
    ]
 
+checkListen :: String -> String
+checkListen l | isIPv6address l = l
+              | isIPv4address l = l
+			  | otherwise         = error "Gitit.checkListen: Not a valid interface name"
+
+getListenOrDefault :: [Opt] -> String
+getListenOrDefault [] = "127.0.0.1"
+getListenOrDefault ((Listen l):_) = l
+getListenOrDefault (_:os) = getListenOrDefault os
+
 parseArgs :: [String] -> IO [Opt]
 parseArgs argv = do
   progname <- getProgName
@@ -139,6 +162,7 @@ handleFlag conf opt = do
     Debug              -> return conf{ debugMode = True }
     Port p             -> return conf{ portNumber = p }
     ConfigFile fname   -> getConfigFromFile fname
+    Listen _           -> return conf
 
 putErr :: ExitCode -> String -> IO a
 putErr c s = B.hPutStrLn stderr (fromString s) >> exitWith c
